﻿using FluentValidation;
using FluentValidation.AspNetCore;
using Hotline.Api.Realtimes;
using Hotline.Application;
using Hotline.Application.Contracts;
using Hotline.NewRock;
using Hotline.Permissions;
using Hotline.Repository.SqlSugar.Extensions;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Serilog;
using Mapster;
using Tr.Sdk;
using XF.Domain.Dependency;
using XF.Domain.Filters;
using XF.Domain.Options;
using XF.EasyCaching;
using XF.Utility.MQ;
using Hotline.Ai.Jths;
using Hotline.Api.Sdk;
using Hotline.CallCenter.Configs;
using Hotline.YbEnterprise.Sdk;
using Hotline.Share.Dtos.SendSms;
using Hotline.Wex;
using Hotline.Application.CallCenter.Calls;
using Hotline.Application.CallCenter;
using Hotline.CallCenter.Calls;
using Swashbuckle.AspNetCore.SwaggerUI;
using Hotline.EventBus;
using MediatR.Pipeline;

namespace Hotline.Api;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://*:50100");
#endif

        services.Configure<IdentityConfiguration>(d => configuration.GetSection(nameof(IdentityConfiguration)).Bind(d));

        var appConfigurationSection = configuration.GetRequiredSection(nameof(AppConfiguration));
        var appConfiguration = appConfigurationSection.Get<AppConfiguration>();
        services.Configure<AppConfiguration>(d => appConfigurationSection.Bind(d));

        var callCenterConfigurationSection = configuration.GetRequiredSection(nameof(CallCenterConfiguration));
        var callCenterConfiguration = callCenterConfigurationSection.Get<CallCenterConfiguration>();
        services.Configure<CallCenterConfiguration>(d => callCenterConfigurationSection.Bind(d));

        //services.Configure<SendSmsConfiguration>(d => configuration.GetSection("SendSms").Bind(d));

        // Add services to the container.
        services
            .BatchInjectServices()
            .RegisterRepository()
            .AddApplication()
            .AddScoped(typeof(IPasswordHasher<>), typeof(PasswordHasher<>))
            ;

        //cache
        services.AddCache(d =>
        {
            var cacheConfig = configuration.GetSection("Cache").Get<CacheOptions>();
            cacheConfig.Adapt(d);
            d.Prefix = "Hotline";
            d.TopicName = "hotline-topic";
        });

        //Authentication
        services.RegisterAuthentication(configuration);

        services.AddControllers(options =>
            {
                options.Filters.Add<UnifyResponseFilter>();
                options.Filters.Add<UserFriendlyExceptionFilter>();
            })
            ;

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        //swagger
        services.RegisterSwagger();

        //signalR
        services.RegisterSignalR(configuration);

        /* CORS */
        services.RegisterCors(configuration, CorsOrigins);

        //mapster
        services.RegisterMapper();

        //mediatr
        services.RegisterMediatR(appConfiguration);

        //app scope
        switch (appConfiguration.AppScope)
        {
            case AppDefaults.AppScope.YiBin:
                //jths 
                services.AddAiJths(configuration.GetSection("AiQuality").Get<AiQualityConfig>().Url);

                var aiVisitConfigSection = configuration.GetSection("AiVisit");
                var aiVisitConfig = aiVisitConfigSection.Get<AiVisitConfig>();
                services.AddAiVisitService(
                    aiVisitConfig.Url,
                    aiVisitConfig.Appkey,
                    aiVisitConfig.ServiceVersion);
                services.Configure<AiVisitConfig>(d => aiVisitConfigSection.Bind(d));

                //宜宾企业服务
                services.Configure<EnterpriseConfig>(d => configuration.GetSection("Enterprise").Bind(d));
                var enterpriseConfig = configuration.GetSection("Enterprise").Get<EnterpriseConfig>();
                services.AddYbEnterpriseSdk(enterpriseConfig?.AddressUrl);
                break;
            case AppDefaults.AppScope.ZiGong:
                break;
            case AppDefaults.AppScope.LuZhou:
                break;
        }

        //callcenter
        var callcenterType = appConfiguration.GetDefaultAppScopeConfiguration().CallCenterType;
        switch (callcenterType)
        {
            case AppDefaults.CallCenterType.XunShi:
                services.AddNewRock(callCenterConfiguration.NewRock);
                break;
            case AppDefaults.CallCenterType.WeiErXin:
                services
                    .AddWex(callCenterConfiguration.Wex)
                    .AddWexDb(configuration);
                break;
            case AppDefaults.CallCenterType.TianRun:
                services
                    .AddScoped<ICallApplication, TianRunCallApplication>()
                    .AddScoped<ITrApplication, TrApplication>()
                    .AddHostedService<CurrentWaitNumService>()
                    .AddHostedService<TelsStatusRefreshService>()
                    .AddTrSdk(callCenterConfiguration.TianRun.Address,
                        callCenterConfiguration.TianRun.Username,
                        callCenterConfiguration.TianRun.Password);
                break;
            case AppDefaults.CallCenterType.XingTang:
                services.AddXingTangDb(callCenterConfiguration.XingTang)
                    .AddScoped<ICallApplication, XingTangCallApplication>()
                    .AddScoped<CallIdManager>()
                    ;
                break;
            default:
                break;
        }


        //sqlsugar
        services.AddSqlSugar(configuration);

        //validator
        services.AddFluentValidationAutoValidation(config =>
        {
            config.DisableDataAnnotationsValidation = true;
        })
            .AddValidatorsFromAssembly(typeof(AppContractsStartupExtensions).Assembly);

        //mq
        services.AddMq(configuration);

        //job
        services.RegisterJob(callcenterType);

        services.AddSingleton<IAuthorizationPolicyProvider, AuthorizationPolicyProvider>();
        services.AddSingleton<IAuthorizationHandler, PermissionHandler>();

        //services.AddScoped<LogFilterAttribute>();
        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        // Configure the HTTP request pipeline.
        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI(c =>
            {
                c.DocExpansion(DocExpansion.None);
                //c.SwaggerEndpoint("/swagger/v1/swagger.json", "My API V1");
            });
        }

        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();
        //app.MapHub<CallCenterHub>("/hubs/callcenter");
        app.MapHub<HotlineHub>("/hubs/hotline");

        app.MapControllers()
            .RequireAuthorization();
        //app.MapSubscribeHandler();

        return app;
    }
}